package Composite_PSO;

/*
	particle.java
	Pablo David Gutiérrez Pérez.
	
	Copyright (c) 2011.

*/

import java.util.ArrayList;
import java.util.Random;

/**
 *
 * @author Pablo David Gutiérrez Pérez
 */
public class de {
	static private int iterations = 15;
	static private int elements = 5;

	/** Scaling Factor */
	static private double F = 0.5;
	/** Crossover rate */
	static private double CR = 0.1;

	/** Population */
	protected ArrayList<ArrayList<Double>> gen;
	/** Fitness of population */
	protected ArrayList<Double> fitness;
	/** Best individual of population */
	protected ArrayList<Double> best;
	/** Best fitness of population */
	protected double best_fitness = Double.MAX_VALUE;
	/** Mutation vector */
	protected ArrayList<Double> mut;

	/** Stop criteria state */
	protected boolean stop = false;
	
	/** Number of evaluations */
	private int ev;

	/** DE constructor */
	public de(){
		best = new ArrayList<Double>();
		mut = new ArrayList<Double>(3);
		fitness = new ArrayList<Double>();;
	}

	/** Initializes the DE population
	 * @param rand Random generator.
	 */
	private void init(Random rand){
		gen = new ArrayList<ArrayList<Double>>();
		for(int i = 0; i < elements; i++){
			ArrayList<Double> aux = new ArrayList<Double>();
			aux.add(rand.nextDouble()*(4-0.1)+0.1);
			aux.add(rand.nextDouble()*(4-0.1)+0.1);
			aux.add(rand.nextDouble()*(1.2-0.4)+0.4);
			gen.add(aux);
		}
	}

	/** DE main function
	 * @param swarm The swarm.
	 * @param g_best The best particle.
	 * @param eval Number of evaluations.
	 * @param max_eval Maximum number of evaluations.
	 * @param rand Random generator.
	 * @param bias Function bias.
	 */
	public int run(ArrayList<particle> swarm,particle g_best, int eval, int max_eval, Random rand, double bias){
		ev = eval;
		//initialization
		init(rand);
		for(int j = 0; j < gen.size() && !stop; j++){
				fitness.add(eval(gen.get(j), swarm, g_best, max_eval, rand, bias));
				if(fitness.get(j) < best_fitness){
					best_fitness = fitness.get(j);
					best = (ArrayList<Double>) gen.get(j).clone();
				}
			}
		
		//Main boucle
		for(int i = 1; i < iterations && ev < max_eval && !stop; i++){
			
			//Mutation & crossover
			ArrayList<ArrayList<Double>> aux = new ArrayList<ArrayList<Double>>();
			for(int j = 0; j < gen.size(); j++){
				mutation(j,rand);
				aux.add(crossover(j, rand));
			}

			//Selection
			for(int j = 0; j < gen.size() && !stop; j++){
				fitness.set(j, eval(aux.get(j), swarm, g_best, max_eval, rand, bias));
				if(fitness.get(j) < best_fitness){
					best_fitness = fitness.get(j);
					best = (ArrayList<Double>) aux.get(j).clone();
				}

			}
			gen = aux;
		}
		return ev-eval;
	}

	/** Evaluates a individual
	 * @param g Individual
	 * @param swarm The swarm.
	 * @param g_best The best particle.
	 * @param max_eval Maximum number of evaluations.
	 * @param rand Random generator.
	 * @param bias Function bias.
	 */
	private double eval(ArrayList<Double> g, ArrayList<particle> swarm, particle g_best, int max_eval, Random rand, double bias){
		double fit = Double.MAX_VALUE;
		ArrayList<particle> copy = (ArrayList<particle>) swarm.clone();
		particle.set(g.get(0), g.get(1), g.get(2));
		for(int k = 0; k < copy.size(); k++)
			copy.get(k).update_velocity(g_best, rand);
		for(int k = 0; k < copy.size() && ev < max_eval && !stop; k++, ev++){
			copy.get(k).update_position();
			if(copy.get(k).fitness < fit){
				fit = copy.get(k).fitness;
				if(copy.get(k).fitness - bias < Math.pow(10, -8)){
					g_best.fitness = copy.get(k).fitness;
					g_best.position = copy.get(k).position.clone();
					stop = true;
				}
			}
		}
		return fit;
	}

	/** Classic DE/Rand/1 mutation
	 * @param index Index of the particle.
	 * @param rand Random generator.
	 */
	private void mutation(int index, Random rand){
		int index1, index2, index3;
		mut.clear();

		//Select first individual
		do{
			index1 = (int) (Math.ceil(rand.nextDouble() * elements) % elements);
		}while(index1 == index);

		//Select second individual
		do{
			index2 = (int) (Math.ceil(rand.nextDouble() * elements) % elements);
		}while(index2 == index1 || index2 == index);

		//Select third individual
		do{
			index3 = (int) (Math.ceil(rand.nextDouble() * elements) % elements);
		}while(index3 == index1 || index3 == index2 || index3 == index);

		//Mutation
		for(int i = 0; i < 3; i++){
			mut.add(gen.get(index1).get(i)+ F*(gen.get(index2).get(i) - gen.get(index3).get(i)));
		}


	}

	/** Classic Binomial crossover
	 * @param index Index of the particle.
	 * @param rand Random generator.
	 */
	private ArrayList<Double> crossover(int index, Random rand){
		int sel = (int) (Math.ceil(rand.nextDouble() * 3)) % 3;
		ArrayList<Double> n = new ArrayList<Double>();
		for(int i = 0; i < 3; i++){
			if(i==sel || rand.nextDouble() <= CR){
				n.add(mut.get(i));
			}
			else{
				n.add(gen.get(index).get(i));
			}
		}
		return n;
	}

}
