package Composite_PSO;

/*
	particle.java
	Pablo David Gutiérrez Pérez.
	
	Copyright (c) 2011.

*/

import java.util.Random;

/**
 *
 * @author Pablo David Gutiérrez Pérez
 */

public class particle {
	/** Cognitive knowledge factor */
	static protected double cognitive = 2;
	/** Social knowledge factor */
	static protected double social = 2;
	/** Inertia factor initial value */
	static protected double inertia = 0.9;
	
	/** Problem's dimension */
	protected int dimension;
	/** Actual position */
	protected double[] position;
	/** best position factor */
	protected double[] best_position;
	/** Velocity vector */
	protected double[] velocity;
	/** Problem bounds */
	protected double max_bounds;
	protected double min_bounds;
	/** Maximum velocity on a component */
	protected double max_velocity;
	/** Fitness of actual position */
	protected double fitness;
	/** Fitness of best position */
	protected double best_fitness;

	/** Default constructor empty */
	public particle(){}
	
	/** Particle constructor
	 * @param dim Problem's dimension
	 * @param max High bound for each dimension
	 * @param min Low bound for each dimension
	 */
	public particle(int dim, double max, double min){
		dimension = dim;
		position = new double[dimension];
		best_position = new double[dimension];
		velocity = new double[dimension];
		max_bounds = max;
		min_bounds = min;
		max_velocity = 0.2*Math.abs(max_bounds - min_bounds);
	}

	/** Initializes a particle position and velocity
	 * @param rand Random generator.
	 */
	public void init(Random rand){
		for(int i = 0; i < dimension; i++){
			position[i] = rand.nextDouble()*(max_bounds - min_bounds) + min_bounds;
			best_position[i] = position[i];
			velocity[i] = rand.nextDouble()*(max_velocity * 2) - max_velocity;
		}
	}

	/** Updates particle's velocity
	 * @param global_best reference particle.
	 * @param rand Random generator.
	 */
	public void update_velocity(particle global_best, Random rand){
		for(int i = 0; i < dimension; i++){
			velocity[i] = (velocity[i] * inertia) + (cognitive * rand.nextDouble() * (best_position[i] - position[i]))
							+ (social * rand.nextDouble() * (global_best.best_position[i] - position[i]));
			if(velocity[i] < -max_velocity)
				velocity[i] = -max_velocity;
			else if(velocity[i] > max_velocity)
				velocity[i] = max_velocity;
		}
	}

	/** Updates particle's position
	 */
	public void update_position(){
		for( int i = 0; i < dimension; i++){
			position[i] = position[i] + velocity[i];
			if(position[i] > max_bounds){
				position[i] = max_bounds;
				velocity[i] = 0;
			}
			else if(position[i] < min_bounds)
			{
				position[i] = min_bounds;
				velocity[i] = 0;
			}
		}
	}

	/** Prints information about the particle
	 * @param n number of particle
	 */
	public void print(int n){
		System.out.println("particle: "+n);
		for(int i = 0; i < position.length; i++){
			System.out.println(position[i]+","+velocity[i]+","+best_position[i]);
		}
		System.out.println(best_fitness + "," + fitness+","+inertia);
	}

	/** Sets the new values for cognitive, social and inertia 
	 * @param new_cognitive new cognitive value
	 * @param new_social new social value
	 * @param new_inertia new inertia value
	 */
	static public void set(double new_cognitive, double new_social, double new_inertia){
		cognitive = new_cognitive;
		social = new_social;
		inertia = new_inertia;
	}
};
