package FIPSO;

/*
	pso.java
	Pablo David Gutiérrez Pérez.
	
	Copyright (c) 2011.

*/

import cec05.benchmark;
import cec05.test_func;
import java.util.ArrayList;
import java.util.Random;

/**
 *
 * @author Pablo David Gutiérrez Pérez
 */
public class pso {
	/** Swarm of particles */
	protected ArrayList<particle> swarm;
	/** Number of evaluations spent */
	protected int eval;
	/** Maximum evaluations */
	protected int max_evaluations;
	/** Problem's dimension */
	protected int dim;
	/** Number of the function */
	protected int fun;
	/** Swarm size */
	protected int size;
	/** Best particle */
	protected particle best_particle;
	/** Random generator */
	protected Random rand;
	/** Type of topology */
	protected int topology;
	/** Type of neighborhood */
	protected int type;
	/** Size of neighborhood */
	protected int neighbor;
	/** Test function */
	protected benchmark test;
	protected test_func function;
	/** Seed of random generator */
	protected long seed;
	/** Best error */
	protected double error;
	/** Type of initialization */
	protected boolean symetric;

	public pso(){}

	/** Function that prints usage
	 */
	private void print_usage(){
		System.out.println("Params error");
		System.out.println("Usage: java -cp dist/PSO.jar FIPSO.pso function dimension seed population type topology symmetric");
		System.out.println("Type 0 -> FIPS");
		System.out.println("Type 1 -> wFIPS");
		System.out.println("Type 2 -> Self");
		System.out.println("Type 3 -> wSelf");
		System.out.println("Topology 0 -> All");
		System.out.println("Topology 1 -> Ring");
		System.out.println("Topology 2 -> Four Clusters");
		System.out.println("Topology 3 -> Pyramid");
		System.out.println("Topology 4 -> Square");
		System.out.println("Topology 5 -> UAll");
		System.out.println("Topology 6 -> URing");
		System.out.println("Topology 7 -> UFour Clusters");
		System.out.println("Topology 8 -> UPyramid");
		System.out.println("Topology 9 -> USquare");
		System.out.println("Symmetric 1 -> symmetric initialization");
		System.out.println("Symmetric 0 -> usual random initialization");
		System.exit(-1);
	}
	
	/** Checks the parameters value
	 */
	private boolean check_parameters(String[] params){
		if(params.length != 7)
			return false;
		int i = 0;
		//Function
		fun = Integer.parseInt(params[i]);
		if(fun <= 0 || fun > 25)
			return false;
		i++;
		//Dimension
		dim = Integer.parseInt(params[i]);
		if(dim <= 0 || dim > 50)
			return false;
		i++;
		//Number of evaluations
		max_evaluations = 10000*dim;
		//Seed
		seed =  Long.parseLong(params[i]);
		rand = new Random(seed);
		i++;
		//Swarm size
		size = Integer.parseInt(params[i]);
		if(size <= 0)
			return false;
		swarm = new ArrayList<particle>(size);
		i++;
		//type 0 -> FIPS; 1 -> wFIPS; 2 -> self; 3 -> wself
		type = Integer.parseInt(params[i]);
		if(type < 0 || type > 3)
			return false;
		i++;
		//topology
		topology = Integer.parseInt(params[i]);
		if(topology < 0 || topology > 9)
			return false;
		i++;
		//symetric
		int aux = Integer.parseInt(params[i]);
		if(aux < 0 || aux > 1)
			return false;
		if(aux == 1)
			symetric = true;
		else
			symetric = false;


		return true;
	}

	/** Calculates the distance between two particles
	 */
	private double distance(particle a, particle b){
		double dist = 0;
		for(int i = 0; i < dim; i++){
			dist += (Math.pow(a.position[i] - b.position[i], 2));
		}

		return Math.sqrt(dist);
	}

	/** Initializes the swarm
	 */
	private void init(){
		eval = 0;
 		test = new benchmark();
		function = test.testFunctionFactory(fun, dim);
		int size_aux;
		size_aux = symetric ? size/2 : size; // if symetric only the first half of the particles uses usual initialization
		for(int i = 0; i < size_aux; i++){
			swarm.add(new particle(dim, test.get_max_value(fun), test.get_min_value(fun)));
			swarm.get(i).init(rand);
			if(fun == 7 || fun == 25){
				swarm.get(i).max_bounds = 999999999;
				swarm.get(i).min_bounds = -999999999;
			}
		}
		if(symetric){
			//The second half of the particles initialization depends of the topology
			if(topology == 3){
				particle aux_1, aux_2;
				aux_1 = swarm.remove(size_aux-1);
				aux_2 = swarm.remove(size_aux-2);
				for(int i = size_aux-2; i < size-4; i++){
					swarm.add(i, new particle(swarm.get(i-(size_aux-2))));
					swarm.get(i).invert(test.get_max_value(fun), test.get_min_value(fun));
				}
				swarm.add(size-4,aux_2);
				swarm.add(size-3,aux_1);
				swarm.add(size-2,new particle(aux_2));
				swarm.add(size-1,new particle(aux_1));
				swarm.get(size-2).invert(test.get_max_value(fun), test.get_min_value(fun));
				swarm.get(size-1).invert(test.get_max_value(fun), test.get_min_value(fun));
			}
			else{
				for(int i = size_aux; i < size; i++){
					swarm.add(i, new particle(swarm.get(i-size_aux)));
					swarm.get(i).invert(test.get_max_value(fun), test.get_min_value(fun));
				}
			}
		}
	}

	/** Return the set of reference particle for a given particle according the topology
	 * @param pos Index of the particle
	 * @return The set of reference particle
	 */
	private ArrayList<particle> reference_particle(int pos){
		ArrayList<particle> a = new ArrayList<particle>();
		switch(topology){
			case 0: //all
			case 5: //Uall
				a.addAll(swarm);
				a.remove(pos);
				break;

			case 1: // ring
			case 6: // Uring
				a.add(swarm.get((pos+1)%swarm.size()));
				//a.add(swarm.get(pos));
				a.add(swarm.get((pos-1+swarm.size())%swarm.size()));
				break;
				
			case 2: // four cluster
			case 7: // Ufour cluster
				int rf = pos;
				while(rf%5!=0){
					rf--;
				}
				switch(pos%5){
					case 0:
						for(int i = 0; i < 5; i++)
							if(rf+i != pos)
								a.add(swarm.get(rf+i));
						a.add(swarm.get((pos+18)%swarm.size()));
						break;
					case 1:
					case 2:
						for(int i = 0; i < 5; i++)
							if(rf+i != pos)
								a.add(swarm.get(rf+i));
						break;
					case 3:
						for(int i = 0; i < 5; i++)
							if(rf+i != pos)
								a.add(swarm.get(rf+i));
						a.add(swarm.get((pos+5)%swarm.size()));
						break;
					case 4:
						for(int i = 0; i < 5; i++)
							if(rf+i != pos)
								a.add(swarm.get(rf+i));
						a.add(swarm.get((pos+14)%swarm.size()));
						break;
				}
				break;
				case 3: // pyramid
				case 8: // Upyramid
					switch (pos){
						case 0:
							a.add(swarm.get(1));
							a.add(swarm.get(4));
							a.add(swarm.get(5));
							break;
						case 1:
							a.add(swarm.get(0));
							a.add(swarm.get(2));
							a.add(swarm.get(4));
							a.add(swarm.get(5));
							a.add(swarm.get(16));
							a.add(swarm.get(17));
							break;
						case 2:
							a.add(swarm.get(1));
							a.add(swarm.get(3));
							a.add(swarm.get(6));
							a.add(swarm.get(7));
							a.add(swarm.get(16));
							a.add(swarm.get(17));
							break;
						case 3:
							a.add(swarm.get(2));
							a.add(swarm.get(6));
							a.add(swarm.get(7));
							break;
						case 4:
							a.add(swarm.get(0));
							a.add(swarm.get(1));
							a.add(swarm.get(5));
							a.add(swarm.get(8));
							a.add(swarm.get(16));
							a.add(swarm.get(18));
							break;
						case 5:
							a.add(swarm.get(0));
							a.add(swarm.get(1));
							a.add(swarm.get(4));
							a.add(swarm.get(10));
							a.add(swarm.get(17));
							a.add(swarm.get(18));
							break;
						case 6:
							a.add(swarm.get(2));
							a.add(swarm.get(3));
							a.add(swarm.get(7));
							a.add(swarm.get(9));
							a.add(swarm.get(16));
							a.add(swarm.get(19));
							break;
						case 7:
							a.add(swarm.get(2));
							a.add(swarm.get(3));
							a.add(swarm.get(6));
							a.add(swarm.get(11));
							a.add(swarm.get(17));
							a.add(swarm.get(19));
							break;
						case 8:
							a.add(swarm.get(4));
							a.add(swarm.get(9));
							a.add(swarm.get(12));
							a.add(swarm.get(13));
							a.add(swarm.get(16));
							a.add(swarm.get(18));
							break;
						case 9:
							a.add(swarm.get(6));
							a.add(swarm.get(8));
							a.add(swarm.get(12));
							a.add(swarm.get(13));
							a.add(swarm.get(16));
							a.add(swarm.get(19));
							break;
						case 10:
							a.add(swarm.get(5));
							a.add(swarm.get(11));
							a.add(swarm.get(14));
							a.add(swarm.get(15));
							a.add(swarm.get(17));
							a.add(swarm.get(18));
							break;
						case 11:
							a.add(swarm.get(7));
							a.add(swarm.get(10));
							a.add(swarm.get(14));
							a.add(swarm.get(15));
							a.add(swarm.get(17));
							a.add(swarm.get(19));
							break;
						case 12:
							a.add(swarm.get(8));
							a.add(swarm.get(9));
							a.add(swarm.get(13));
							break;
						case 13:
							a.add(swarm.get(8));
							a.add(swarm.get(9));
							a.add(swarm.get(12));
							a.add(swarm.get(14));
							a.add(swarm.get(18));
							a.add(swarm.get(19));
							break;
						case 14:
							a.add(swarm.get(10));
							a.add(swarm.get(11));
							a.add(swarm.get(13));
							a.add(swarm.get(15));
							a.add(swarm.get(18));
							a.add(swarm.get(19));
							break;
						case 15:
							a.add(swarm.get(10));
							a.add(swarm.get(11));
							a.add(swarm.get(15));
							break;
						case 16:
							a.add(swarm.get(1));
							a.add(swarm.get(2));
							a.add(swarm.get(4));
							a.add(swarm.get(6));
							a.add(swarm.get(8));
							a.add(swarm.get(9));
							break;
						case 17:
							a.add(swarm.get(1));
							a.add(swarm.get(2));
							a.add(swarm.get(5));
							a.add(swarm.get(7));
							a.add(swarm.get(10));
							a.add(swarm.get(11));
							break;
						case 18:
							a.add(swarm.get(4));
							a.add(swarm.get(5));
							a.add(swarm.get(8));
							a.add(swarm.get(10));
							a.add(swarm.get(13));
							a.add(swarm.get(14));
							break;
						case 19:
							a.add(swarm.get(6));
							a.add(swarm.get(7));
							a.add(swarm.get(9));
							a.add(swarm.get(11));
							a.add(swarm.get(13));
							a.add(swarm.get(14));
							break;
					}
					break;
				case 4: // square
				case 9: // Usquare
					int sum = 0;
					int sum2 = 0;
					if(pos >= 5 && pos < 10)
						sum = 5;
					else if(pos >= 10 && pos < 15)
						sum = 10;
					else if(pos >= 15)
						sum = 15;
					
					if(pos >= 10)
						sum2 = 10;
					
					a.add(swarm.get(((pos + 1)%5 + sum)));
					a.add(swarm.get(((pos - 1 + 5)%5 + sum)));
					a.add(swarm.get(((pos + 5)%10 + sum2)));
					a.add(swarm.get(((pos + 10)%20)));
		}
		return a;
	}

	/** PSO Main function */
	public void run_pso(){
		/* Evaluates the initial population and find the first best*/
		best_particle = new particle();
		swarm.get(0).fitness = function.f(swarm.get(0).position);
		eval++;
		swarm.get(0).best_fitness = swarm.get(0).fitness;
		swarm.get(0).best_position = swarm.get(0).position;
		best_particle.best_fitness = swarm.get(0).best_fitness;
		best_particle.best_position = swarm.get(0).best_position.clone();

		for(int i = 1; i < swarm.size() && eval < max_evaluations; i++){
			swarm.get(i).fitness = function.f(swarm.get(i).position);
			eval++;
			swarm.get(i).best_fitness = swarm.get(i).fitness;
			swarm.get(i).best_position = swarm.get(i).position.clone();
			if(swarm.get(i).best_fitness < best_particle.best_fitness)
				best_particle = swarm.get(i);
		}

		error = best_particle.best_fitness - function.bias();

		/* PSO main boucle*/
		while(eval < max_evaluations && error > Math.pow(10, -8)){
			/* Look for the worse value for weighted mode */
			double worse=-Double.MAX_VALUE;
			if(type %2 == 1)
				for(int i = 0; i < swarm.size(); i++){
					if(swarm.get(i).best_fitness > worse)
						worse = swarm.get(i).best_fitness;
				}
			
			/* Updates particles' velocity */
			for(int i = 0; i < swarm.size(); i++){
				swarm.get(i).update_velocity(reference_particle(i), rand, type, topology, best_particle.best_fitness/*function.bias()/**/,worse);
			}

			/* Updates particles' position */
			for(int i = 0; i < swarm.size() && eval < max_evaluations; i++){
				swarm.get(i).update_position();
				/* Evaluates new positions*/
				swarm.get(i).fitness = function.f(swarm.get(i).position);
				eval++;
				
				/* Check for own best */
				if(swarm.get(i).fitness < swarm.get(i).best_fitness){
					swarm.get(i).best_fitness = swarm.get(i).fitness;
					swarm.get(i).best_position = swarm.get(i).position.clone();
				}
				
				/* Check for best particle*/
				if(swarm.get(i).fitness < best_particle.best_fitness){
					best_particle = swarm.get(i);
				}
			}
			error = best_particle.best_fitness - function.bias();
		}
	}

	/** Print results
	 */
	public void print_result(){
		System.out.println(seed + "," + fun + "," + function.bias() + ","+
				best_particle.best_fitness + "," + error + "," + eval);
	}

	/** Main function
	 */
	static public void main(String[] args){
		pso PSO = new pso();
		if(!PSO.check_parameters(args))
			PSO.print_usage();

		PSO.init();

	  //long t0 = System.currentTimeMillis();
		PSO.run_pso();
	  //long t1 = System.currentTimeMillis();
	  
	  //System.out.println(t1-t0);

		PSO.print_result();
	}

}
