package Frankensteing_PSO;

/*
	frankenstein_pso.java
	Pablo David Gutiérrez Pérez.
	
	Copyright (c) 2011.

*/

import cec05.benchmark;
import cec05.test_func;
import java.util.ArrayList;
import java.util.Random;

/**
 *
 * @author Pablo David Gutiérrez Pérez
 */
public class frankensteing_pso {
	/** Swarm of particles */
	protected ArrayList<particle> swarm;
	/** Number of evaluations spent */
	protected int eval;
	/** Maximum evaluations */
	protected int max_evaluations;
	/** Problem's dimension */
	protected int dim;
	/** Number of the function */
	protected int fun;
	/** Swarm size */
	protected int size;
	/** Best particle */
	protected particle best_particle;
	/** Random generator */
	protected Random rand;
	/** Type of neighborhood */
	protected int type;
	/** Size of neighborhood */
	protected int neighbor;
	/** Test function */
	protected benchmark test;
	protected test_func function;
	/** Seed of random generator */
	protected long seed;
	/** Best error */
	protected double error;
	/** Number of iterations */
	protected int iterations;
	/** Maximum iterations to update inertia */
	protected int max_iterations_inertia;
	/** Maximum iterations to update topology */
	protected int max_iterations_topology;
	/** Topology */
	protected ArrayList<ArrayList<particle> > connections;
	/** Ensure the number of connections removed follows an arithmetic regression */
	protected int esteps;

	public frankensteing_pso(){}

	/** Function that prints usage
	 */
	private void print_usage(){
		System.out.println("Params error");
		System.out.println("Usage: java -cp PSO.jar dist/Frankensteing_PSO.frankensteing_pso function dimension seed population top_type ine_type");
		System.out.println("top_type -> sets max_iterations_topology to size*(top_type+1)");
		System.out.println("ine_type -> sets max_iterations_intertia to size*size*(ine_type+1)");
		System.exit(-1);
	}
	
	/** Checks the parameters value
	 */
	private boolean check_parameters(String[] params){
		if(params.length != 5 && params.length != 6)
			return false;
		int i = 0;
		//Function
		fun = Integer.parseInt(params[i]);
		if(fun <= 0 || fun > 25)
			return false;
		i++;
		//Dimension
		dim = Integer.parseInt(params[i]);
		if(dim <= 0 || dim > 50)
			return false;
		i++;
		//Number of evaluations
		max_evaluations = 10000*dim;
		//Seed
		seed =  Long.parseLong(params[i]);
		rand = new Random(seed);
		i++;
		//Swarm size
		size = Integer.parseInt(params[i]);
		if(size <= 0)
			return false;
		swarm = new ArrayList<particle>(size);
		i++;
		
		int top_type = Integer.parseInt(params[i]);
		if(top_type < 0 )
			return false;
		
		max_iterations_topology = size * (top_type+1);
		i++;
		
		int ine_type = Integer.parseInt(params[i]);
		if(ine_type < 0 )//|| ine_type > 3)
			return false;
		
		max_iterations_inertia = size * size * (ine_type+1);

		return true;
	}

	/** Calculates the distance between two particles
	 */
	private double distance(particle a, particle b){
		double dist = 0;
		for(int i = 0; i < dim; i++){
			dist += (Math.pow(a.position[i] - b.position[i], 2));
		}

		return Math.sqrt(dist);
	}

	/** Initializes the swarm using symmetric initialization
	 */
	private void init(){
		eval = 0;
 		test = new benchmark();
		function = test.testFunctionFactory(fun, dim);
		int size_aux;
		/* Usual initialization for the first half */
		size_aux =  size/2;
		for(int i = 0; i < size_aux; i++){
			swarm.add(new particle(dim, test.get_max_value(fun), test.get_min_value(fun)));
			swarm.get(i).init(rand);
			if(fun == 7 || fun == 25){
				swarm.get(i).max_bounds = 999999999;
				swarm.get(i).min_bounds = -999999999;
			}
		}
		
		/* Symmetric initialization for the second half */
		for(int i = size_aux; i < size; i++){
			swarm.add(i, new particle(swarm.get(i-size_aux)));
			swarm.get(i).invert(test.get_max_value(fun), test.get_min_value(fun));
		}
		iterations = 0;
		esteps = 0;
		
		/* Topology fully conected */
		connections = new ArrayList<ArrayList<particle>>();
		for(int i = 0; i < swarm.size(); i++){
			connections.add(new ArrayList<particle>(swarm));
			connections.get(i).remove(i);
		}
	}

	/** Return the set of reference particle for a given particle
	 * @param pos Index of the particle
	 * @return The set of reference particle
	 */
	private ArrayList<particle> reference_particle(int pos){
		return connections.get(pos);
	}
	
	/** Updates the topology removing some connections
	 * @param iter Actual iterations
	 * @param max_iter Number of iterations to finish the update
	 */
	private void update_connections(int iter, int max_iter){
		/* Check if the algorithm needs to update the topology */
		if(iter <= max_iter && iter % Math.ceil((max_iter*1.0)/(swarm.size()-3)) == 0){
			
			/* remove swarm.size() - (2+esteps) connections*/
			for(int i = 0; i < swarm.size() - (2+esteps); i++){
				if(connections.get(i).size() > 2){
					int sel, prev, post;
					prev = ((i - 1)+swarm.size()) % swarm.size();
					post = (i + 1) % swarm.size();
					do{
						sel = rand.nextInt(swarm.size());
					}while(sel == i || sel == prev || sel == post || !connections.get(i).contains(swarm.get(sel)));
					connections.get(i).remove(swarm.get(sel));
					connections.get(sel).remove(swarm.get(i));
				}
			}
			esteps++;
		}
	}

	/** PSO Main function */
	public void run_pso(){
		/* Evaluates the initial population and find the first best*/
		best_particle = new particle();
		swarm.get(0).fitness = function.f(swarm.get(0).position);
		eval++;
		swarm.get(0).best_fitness = swarm.get(0).fitness;
		swarm.get(0).best_position = swarm.get(0).position.clone();
		best_particle.best_fitness = swarm.get(0).best_fitness;
		best_particle.best_position = swarm.get(0).best_position.clone();

		for(int i = 1; i < swarm.size() && eval < max_evaluations; i++){
			swarm.get(i).fitness = function.f(swarm.get(i).position);
			eval++;
			swarm.get(i).best_fitness = swarm.get(i).fitness;
			swarm.get(i).best_position = swarm.get(i).position.clone();
			if(swarm.get(i).best_fitness < best_particle.best_fitness)
				best_particle = swarm.get(i);
		}
		iterations = 1;
		error = best_particle.best_fitness - function.bias();
		
		update_connections(iterations, max_iterations_topology);
		particle.update_intertia(iterations, max_iterations_inertia);

		/* PSO main boucle*/
		while(eval < max_evaluations && error > Math.pow(10, -8)){
			/* Updates particles' velocity */
			for(int i = 0; i < swarm.size(); i++){
				swarm.get(i).update_velocity(reference_particle(i), rand);
			}

			/* Updates particles' position */
			for(int i = 0; i < swarm.size() && eval < max_evaluations; i++){
				swarm.get(i).update_position();
				/* Evaluates new positions*/
				swarm.get(i).fitness = function.f(swarm.get(i).position);
				eval++;
				
				/* Check for own best */
				if(swarm.get(i).fitness < swarm.get(i).best_fitness){
					swarm.get(i).best_fitness = swarm.get(i).fitness;
					swarm.get(i).best_position = swarm.get(i).position.clone();
				}
				
				/* Check for best particle*/
				if(swarm.get(i).fitness < best_particle.best_fitness)
					best_particle = swarm.get(i);
			}
			iterations++;
			error = best_particle.best_fitness - function.bias();
			
			/* Update topology */
			update_connections(iterations, max_iterations_topology);
			
			/* Update inertia */
			particle.update_intertia(iterations, max_iterations_inertia);
		}
	}

	/** Print results
	 */
	public void print_result(){
		System.out.println(seed + "," + fun + "," + function.bias() + ","+
				best_particle.best_fitness + "," + error + "," + eval);
	}

	/** Main function
	 */
	static public void main(String[] args){
		frankensteing_pso PSO = new frankensteing_pso();
		if(!PSO.check_parameters(args))
			PSO.print_usage();

		PSO.init();

		PSO.run_pso();

		PSO.print_result();
	}

}
