package Frankensteing_PSO;

/*
	particle.java
	Pablo David Gutiérrez Pérez.
	
	Copyright (c) 2011.

*/

import java.util.ArrayList;
import java.util.Random;

/**
 *
 * @author Pablo David Gutiérrez Pérez
 */

public class particle {
	/** Cognitive knowledge factor */
	private static double cognitive = 2;
	/** Social knowledge factor */
	private static double social = 2;
	/** Inertia factor initial value */
	private static double inertia = 0.9;
	
	/** Problem's dimension */
	protected int dimension;
	/** Actual position */
	protected double[] position;
	/** best position factor */
	protected double[] best_position;
	/** Velocity vector */
	protected double[] velocity;
	/** Problem bounds */
	protected double max_bounds;
	protected double min_bounds;
	/** Maximum velocity on a component */
	protected double max_velocity;
	/** Fitness of actual position */
	protected double fitness;
	/** Fitness of best position */
	protected double best_fitness;

	/** Default constructor empty */
	public particle(){}
	
	/** Particle constructor
	 * @param dim Problem's dimension
	 * @param max High bound for each dimension
	 * @param min Low bound for each dimension
	 */
	public particle(int dim, double max, double min){
		dimension = dim;
		position = new double[dimension];
		best_position = new double[dimension];
		velocity = new double[dimension];
		max_bounds = max;
		min_bounds = min;
		//max_velocity = 0.2*Math.abs(max_bounds - min_bounds);
		max_velocity = Math.max(Math.abs(max_bounds), Math.abs(min_bounds));
	}
	
	/** Copy constructor, makes a new particle equal to another given
	 * @param base Particle to copy.
	 */
	public particle(particle base){
		dimension = base.dimension;
		position = base.position.clone();
		best_position = base.best_position.clone();
		velocity = base.velocity.clone();
		max_bounds = base.max_bounds;
		min_bounds = base.min_bounds;
		max_velocity = base.max_velocity;
	}
		
	/** Invert the position and velocity of a particle
	 * @param max High bound.
	 * @param min Low bound.
	 */
	public void invert(double max, double min){
		double mid = (max - min) / 2;
		double dif;
		for(int i = 0; i < position.length; i++){
			dif = Math.abs(mid - position[i]);
			if(position[i] < mid)
				position[i] = mid + dif;
			else
				position[i] = mid - dif;
			best_position[i] = position[i];
		}
		for(int i = 0; i < velocity.length; i++)
			velocity[i] = -1.0 * velocity[i];
	}

	/** Initializes a particle position and velocity
	 * @param rand Random generator.
	 */
	public void init(Random rand){
		for(int i = 0; i < dimension; i++){
			position[i] = rand.nextDouble()*(max_bounds - min_bounds) + min_bounds;
			best_position[i] = position[i];
			velocity[i] = rand.nextDouble()*(max_velocity * 2) - max_velocity;
		}
	}
	
	/** Calculate the reference position to update the velocity
	 * @param environment Environment.
	 * @param rand Random generator.
	 */
	private double[] reference_position(ArrayList<particle> environment, Random rand){
	
		double ref[] = new double[dimension];
		double coef[] = new double[environment.size()];
		double w[] = new double[environment.size()];
		for(int i = 0; i < ref.length; i++)
			ref[i] = 0;
		
		/** Generate the random coefficients  */
		for(int i = 0; i < environment.size(); i++)
			coef[i] = rand.nextDouble()*((social+cognitive)/environment.size());
		
		//Calc w
		// Every particle on environment has the same weight
		for(int i = 0; i < environment.size(); i++)
			w[i] = 1.0;
		
		double div = 0;
		for(int i = 0; i < environment.size(); i++){
			for(int j = 0; j < ref.length; j++)
				ref[j] += w[i]*coef[i]*environment.get(i).best_position[j];
			div += w[i]*coef[i];
		}
		for(int i = 0; i < ref.length; i++)
			ref[i] = ref[i]/div;
		
		return ref;
	}
	
	/** Updates particle's velocity
	 * @param global_best reference particle.
	 * @param rand Random generator.
	 */
	public void update_velocity(ArrayList<particle> environment, Random rand){
		double reference[];
		reference = reference_position(environment, rand);
		for(int i = 0; i < dimension; i++){
			velocity[i] = inertia * (velocity[i] + ((cognitive + social)*(reference[i] - position[i])));
			if(velocity[i] < -max_velocity)
				velocity[i] = -max_velocity;
			else if(velocity[i] > max_velocity)
				velocity[i] = max_velocity;
		}
	}

	/** Updates particle's position
	 */
	public void update_position(){
		for( int i = 0; i < dimension; i++){
			position[i] = position[i] + velocity[i];
			if(position[i] > max_bounds){
				position[i] = max_bounds;
				velocity[i] = 0;
			}
			else if(position[i] < min_bounds)
			{
				position[i] = min_bounds;
				velocity[i] = 0;
			}
		}
	}

	/** Updates inertia value for all particles
	 * @param iter number of iterations
	 * @param max_iter maximum iterations to update inertia
	 */
	static void update_intertia(int iter, int max_iter){
		if(iter < max_iter)
			inertia = (((max_iter - iter)*1.0)/max_iter)*0.5 + 0.4;
		else
			inertia = 0.4;
	}

	/** Prints information about the particle
	 * @param n number of particle
	 */
	public void print(int n){
		System.out.println("particle: "+n);
		for(int i = 0; i < position.length; i++){
			System.out.println(position[i]+","+velocity[i]+","+best_position[i]);
		}
		System.out.println(best_fitness + "," + fitness+","+inertia);
	}
};
