package H_PSO;

/*
	h_pso.java
	Pablo David Gutiérrez Pérez.
	
	Copyright (c) 2011.

*/

import cec05.benchmark;
import cec05.test_func;
import java.util.ArrayList;
import java.util.Random;

/**
 *
 * @author Pablo David Gutiérrez Pérez
 */
public class h_pso {
	/** Swarm of particles */
	protected ArrayList<particle> swarm;
	/** Number of evaluations spent */
	protected int eval;
	/** Maximum evaluations */
	protected int max_evaluations;
	/** Problem's dimension */
	protected int dim;
	/** Number of the function */
	protected int fun;
	/** Swarm size */
	protected int size;
	/** Best particle */
	protected particle best_particle;
	/** Random generator */
	protected Random rand;
	/** Type of neighborhood */
	protected int type;
	/** Size of neighborhood */
	protected int neighbor;
	/** Test function */
	protected benchmark test;
	protected test_func function;
	/** Seed of random generator */
	protected long seed;
	/** Best error */
	protected double error;
	/** First particle of the hierarchy */
	protected particle root;
	/** Number of sons */
	protected int d;
	/** Number of levels */
	protected int h;
	/** Number of son decreased when adapt */
	protected int k;
	/** Frequency of adaptation */
	protected int f;
	/** Minimum d value */
	protected int d_min;

	public h_pso(){}

	/** Function that prints usage
	 */
	private void print_usage(){
		System.out.println("Params error");
		System.out.println("Usage: java -cp dist/PSO.jar -cp H_PSO.h_pso function dimension seed population d h type");
		System.out.println("d -> Number of sons per node");
		System.out.println("h -> Heigh of the tree");
		System.out.println("Type 0 -> HPSO");
		System.out.println("Type 1 -> AHPSO");
		System.out.println("Type 2 -> HVPSO");
		System.out.println("Type 3 -> H^PSO");
		System.exit(-1);
	}
	
	/** Checks the parameters value
	 */
	private boolean check_parameters(String[] params){
		if(params.length != 7 && params.length != 10)
			return false;
		int i = 0;
		//Function
		fun = Integer.parseInt(params[i]);
		if(fun <= 0 || fun > 25)
			return false;
		i++;
		//Dimension
		dim = Integer.parseInt(params[i]);
		if(dim <= 0 || dim > 50)
			return false;
		i++;
		//Number of evaluations
		max_evaluations = 10000*dim;
		//Seed
		seed =  Long.parseLong(params[i]);
		rand = new Random();
		rand.setSeed(seed);
		i++;
		//Swarm size
		size = Integer.parseInt(params[i]);
		if(size <= 0)
			return false;
		swarm = new ArrayList<particle>(size);
		i++;
		//number of sons by node
		d = Integer.parseInt(params[i]);
		i++;
		//number of tree levels
		h = Integer.parseInt(params[i]);
		i++;
		//type of algorithm
		//0 -> hpso
		//1 -> ahpso
		//2 -> V hpso
		//3 -> ^ hpso
		type = Integer.parseInt(params[i]);
		if(type < 0 || type > 3)
			return false;
		i++;
		if(type == 1){
			if(params.length == 7)
				return false;
			k = Integer.parseInt(params[i]);		//step
			if(k < 0)
				return false;
			i++;
			f = Integer.parseInt(params[i]);		//frecuency
			if(f < 0)
				return false;
			i++;
			d_min = Integer.parseInt(params[i]);	// minimal d
			if(d_min < 0)
				return false;
			i++;
		}

		return true;
	}

	/** Calculates the distance between two particles
	 */
	private double distance(particle a, particle b){
		double dist = 0;
		for(int i = 0; i < dim; i++){
			dist += (Math.pow(a.position[i] - b.position[i], 2));
		}

		return Math.sqrt(dist);
	}

	/** Initializes the swarm
	 */
	private void init(){
		eval = 0;
 		test = new benchmark();
		function = test.testFunctionFactory(fun, dim);
		for(int i = 0; i < size; i++){
			swarm.add(new particle(dim, test.get_max_value(fun), test.get_min_value(fun)));
			swarm.get(i).init(rand);
			swarm.get(i).number = i;
			if(fun == 7 || fun == 25){
				swarm.get(i).max_bounds = Double.MAX_VALUE;
				swarm.get(i).min_bounds = -Double.MAX_VALUE;
			}
		}
	}
	
	/** Initializes the hierarchy according to the initial fitness
	 */
	private void init_tree(){
		/* Sort the particles */
		ArrayList<particle> order;
		order = new ArrayList<particle>();
		order.add(swarm.get(0));
		boolean inserted;
		for(int i = 1; i < swarm.size(); i++){
			inserted = false;
			for(int j = 0; j < order.size() && ! inserted; j++)
				if(swarm.get(i).fitness < order.get(j).fitness){
					inserted = true;
					order.add(j, swarm.get(i));
				}
			if(inserted == false)
				order.add(swarm.get(i));
		}
		
		/* Construct the hierarchy */
		root = order.remove(0);
		ArrayList<particle> pend;
		pend = new ArrayList<particle>();
		pend.add(root);
		root.level = 0;
		int level = 0;
		while(order.size() > 0 && level < h){
			particle actual = pend.remove(0);
			level = actual.level+1;
			if(level >= h){
				pend.add(0, actual);
				break;
			}
			for(int i = 0; i < d && order.size() > 0; i++)
				actual.children.add(order.remove(0));
			pend.addAll(actual.children);
			for(int i = 0; i < actual.children.size(); i++){
				actual.children.get(i).parent = actual;
				actual.children.get(i).level = level;
			}
		}
		
		/* if there are mor nodes than hierarchy parameters suport add them as leafs*/
		if(order.size() > 0)
			while(order.size() > 0)
				for(int i = 0; i < pend.size() && order.size() > 0; i++){
					pend.get(i).children.add(order.remove(0));
					pend.get(i).children.get(pend.get(i).children.size()-1).parent = pend.get(i);
					pend.get(i).children.get(pend.get(i).children.size()-1).level = pend.get(i).level + 1;
				}
	}
	
	/** Updates the hierarchy */
	private void update_tree(){
		ArrayList<particle> pend;
		pend = new ArrayList<particle>();
		/* Start from root */
		pend.add(root);
		while(pend.size() > 0){
			particle actual = pend.remove(0);
			if(actual.children.size() > 0){
				/* Look for the best children */
				int pos_min = 0;
				for(int i = 1; i < actual.children.size() ; i++)
					if(actual.children.get(i).fitness < actual.children.get(pos_min).fitness)
						pos_min = i;
				if(actual.children.get(pos_min).fitness < actual.fitness){
					//if the best children is better than its father
					particle selected = actual.children.remove(pos_min);
					//change actual.parent.children to selected
					if(actual.parent != null){
						actual.parent.children.remove(actual);
						actual.parent.children.add(selected);
					}
					
					//exchange parents actual an selected position
					selected.parent = actual.parent;
					actual.parent = selected;
					
					//change parent on selected children
					for(int i = 0; i < selected.children.size();i++)
						selected.children.get(i).parent = actual;
					
					//change parent on actual children
					for(int i = 0; i < actual.children.size();i++)
						actual.children.get(i).parent = selected;
					actual.children.add(pos_min, actual);
					
					//change level
					selected.level--;
					actual.level++;
					
					//exchange children
					ArrayList<particle> auxiliar = actual.children;
					actual.children = selected.children;
					selected.children = auxiliar;
					
					//change actual
					actual = actual.parent;
				}
				/* Add the children to the list */
				pend.addAll(0, actual.children);
				if(actual.level == 0)
					root = actual;
			}
		}
	}

	/** Gets all the particles below another given
	 * @param p The particle
	 * @return All the particles below p
	 */
	private ArrayList<particle> get_subtree(particle p){
		if(p.children.isEmpty())
			return new ArrayList<particle>();
		else{
			ArrayList<particle> ret = new ArrayList<particle>();
			ret.addAll(p.children);
			for(int i = 0; i < p.children.size(); i++)
				ret.addAll(get_subtree(p.children.get(i)));
			return ret;
		}
	}
	
	/** Prints the hierarchy */
	private void print_tree(){
		for(int i = 0; i < swarm.size(); i++){
			System.err.println(swarm.get(i).number + " " + swarm.get(i).level + " " + swarm.get(i) + " " + swarm.get(i).parent + " " + swarm.get(i).children.size());
		}
		System.err.println();
		System.err.println();
	}
	
	/** Checks if the hierarchy has to be adapted and, in case, adapt it
	 * @param iterations The number of iterations
	 */
	private void adapt_tree(int iterations){
		/* Check if the algorithm is using adaptative mode and the adaptation condition  */
		if(type == 1 && (iterations % f) == 0){
			//print_tree();
			int new_d = d > d_min ? d - k : d;
			/* The adaptation is done by steps of one degree */
			for(; d > new_d; d--){
				particle actual;
				ArrayList<particle> pend = new ArrayList<particle>();
				ArrayList<particle> discard = new ArrayList<particle>();
				ArrayList<particle> leaf = new ArrayList<particle>();
				
				/* Start from the root */
				pend.add(root);
				while(pend.size() > 0){
					actual = pend.remove(0);
					/*Check the adapt conditions*/
					if(actual.children.size() > new_d && actual.level < h){
						/*Remove the best subtree*/
						int best = 0;
						for(int i = 1; i < actual.children.size(); i++)
							if(actual.children.get(i).fitness < actual.children.get(best).fitness)
								best = i;

						discard.add(actual.children.remove(best));
						discard.addAll(get_subtree(discard.get(discard.size()-1)));

						pend.addAll(actual.children);
					}
					/* if doesn't fit adapt conditions check if the node is a leaf */
					else if(actual.children.size() < d)
						leaf.add(actual);
				}

				/* Clear the previos hierarchy on discard nodes */
				for(int i = 0; i < discard.size(); i++)
					discard.get(i).children.clear();

				/* Adds the discard nodes to leafs*/
				int level = 0;
				while(discard.size() > 0 && level < h){
					actual = leaf.remove(0);
					level = actual.level+1;
					if(level >= h){
						leaf.add(0, actual);
						break;
					}
					for(int i = actual.children.size(); i < new_d && discard.size() > 0; i++){
						actual.children.add(discard.remove(0));
						actual.children.get(actual.children.size()-1).parent = actual;
						actual.children.get(actual.children.size()-1).level = level;
					}
					leaf.addAll(actual.children);
				}
				if(discard.size() > 0){
					while(discard.size() > 0){
						for(int i = 0; i < leaf.size() && discard.size() > 0; i++){
							leaf.get(i).children.add(discard.remove(0));
							leaf.get(i).children.get(leaf.get(i).children.size()-1).parent = leaf.get(i);
							leaf.get(i).children.get(leaf.get(i).children.size()-1).level = leaf.get(i).level + 1;
						}
					}
				}
			}
			//print_tree();
		}
	}
	
	/** Return the reference particle for a given particle
	 * @param pos Index of the particle
	 * @return The reference particle
	 */
	private particle reference_particle(int pos){
		if(swarm.get(pos).parent!=null)
			return swarm.get(pos).parent;
		else
			return swarm.get(pos);
				
	}

	/** PSO Main function */
	public void run_pso(){
		/* Evaluates the initial population and find the first best*/
		int iterations = 0;
		best_particle = new particle();
		swarm.get(0).fitness = function.f(swarm.get(0).position);
		eval++;
		swarm.get(0).best_fitness = swarm.get(0).fitness;
		swarm.get(0).best_position = swarm.get(0).position.clone();
		best_particle.best_fitness = swarm.get(0).best_fitness;
		best_particle.best_position = swarm.get(0).best_position.clone();

		for(int i = 1; i < swarm.size() && eval < max_evaluations; i++){
			swarm.get(i).fitness = function.f(swarm.get(i).position);
			eval++;
			swarm.get(i).best_fitness = swarm.get(i).fitness;
			swarm.get(i).best_position = swarm.get(i).position.clone();
			if(swarm.get(i).best_fitness < best_particle.best_fitness)
				best_particle = swarm.get(i);
		}
		
		/* Initialize the hierarchy */
		init_tree();
		
		error = best_particle.best_fitness - function.bias();
		/* Inertia update */
		for(int i = 0; i < swarm.size(); i++)
			swarm.get(i).update_intertia(eval, max_evaluations, type,h);

		/* PSO main boucle*/
		while(eval < max_evaluations && error > Math.pow(10, -8)){
			iterations++;
			/* Updates particles' velocity */
			for(int i = 0; i < swarm.size(); i++){
				swarm.get(i).update_velocity(reference_particle(i), rand);
			}

			/* Updates particles' position */
			for(int i = 0; i < swarm.size() && eval < max_evaluations; i++){
				swarm.get(i).update_position();
				/* Evaluates new positions*/
				swarm.get(i).fitness = function.f(swarm.get(i).position);
				eval++;
				
				/* Check for own best */
				if(swarm.get(i).fitness < swarm.get(i).best_fitness){
					swarm.get(i).best_fitness = swarm.get(i).fitness;
					swarm.get(i).best_position = swarm.get(i).position.clone();
				}
				
				/* Check for best particle*/
				if(swarm.get(i).fitness < best_particle.best_fitness)
					best_particle = swarm.get(i);
			}
			error = best_particle.best_fitness - function.bias();
			
			/* Update the hierarchy */
			update_tree();
			
			/* Adapt the hierarchy, if we are on adaptative mode */
			adapt_tree(iterations);
			
			/* Inertia update */
			for(int i = 0; i < swarm.size(); i++)
				swarm.get(i).update_intertia(eval, max_evaluations, type, h);
		}
	}

	/** Print results
	 */
	public void print_result(){
		System.out.println(seed + "," + fun + "," + function.bias() + ","+
				best_particle.best_fitness + "," + error + "," + eval);
	}

	/** Main function
	 */
	static public void main(String[] args){
		h_pso PSO = new h_pso();
		if(!PSO.check_parameters(args))
			PSO.print_usage();

		PSO.init();

	  //long t0 = System.currentTimeMillis();
		PSO.run_pso();
	  //long t1 = System.currentTimeMillis();
	  
	  //System.out.println(t1-t0);

		PSO.print_result();
	}

}
