package OLPSO;

/*
	olpso.java
	Pablo David Gutiérrez Pérez.
	
	Copyright (c) 2011.

*/

import cec05.benchmark;
import cec05.test_func;
import java.util.ArrayList;
import java.util.Random;

/**
 *
 * @author Pablo David Gutiérrez Pérez
 */
public class olpso {
	/** Swarm of particles */
	protected ArrayList<particle> swarm;
	/** Number of evaluations spent */
	protected int eval;
	/** Maximum evaluations */
	protected int max_evaluations;
	/** Problem's dimension */
	protected int dim;
	/** Number of the function */
	protected int fun;
	/** Swarm size */
	protected int size;
	/** Best particle */
	protected particle best_particle;
	/** Random generator */
	protected Random rand;
	/** Type of neighborhood */
	protected int type;
	/** Size of neighborhood */
	protected int neighbor;
	/** Test function */
	protected benchmark test;
	protected test_func function;
	/** Seed of random generator */
	protected long seed;
	/** Best error */
	protected double error;
	/** Orthogonal array */
	protected int L[][];
	/** Maximum number of iterations without find a better position */
	protected int max_gen;

	public olpso(){}

	/** Function that prints usage
	 */
	private void print_usage(){
		System.out.println("Params error");
		System.out.println("Usage: java -cp dist/PSO.jar OLPSO.olpso function dimension seed population type neighbor max_gen");
		System.out.println("Type 0 -> global, doesn't need neighbor param");
		System.out.println("Type 1 -> local social");
		System.out.println("Type 2 -> local geographical");
		System.out.println("max_gen -> number of generations to wait for regenerate the artificial particle");
		System.exit(-1);
	}
	
	/** Checks the parameters value
	 */
	private boolean check_parameters(String[] params){
		if(params.length != 7 && params.length != 6)
			return false;
		int i = 0;
		//Function
		fun = Integer.parseInt(params[i]);
		if(fun <= 0 || fun > 25)
			return false;
		i++;
		//Dimension
		dim = Integer.parseInt(params[i]);
		if(dim <= 0 || dim > 50)
			return false;
		i++;
		//Number of evaluations
		max_evaluations = 10000*dim;
		//Seed
		seed =  Long.parseLong(params[i]);
		rand = new Random(seed);
		i++;
		//Swarm size
		size = Integer.parseInt(params[i]);
		if(size <= 0)
			return false;
		swarm = new ArrayList<particle>(size);
		i++;
		//type of algorithm
		// 0 -> global
		// 1 -> social
		// 2 -> geographical
		type = Integer.parseInt(params[i]);
		if(type < 0 || type >= 3)
			return false;
		i++;
		if(type > 0){
			if(params.length == 6)
				return false;
			neighbor = Integer.parseInt(params[i]);
			if(neighbor <= 0 || neighbor > size)
				return false;
			i++;
		}
		max_gen = Integer.parseInt(params[i]);
		if(max_gen < 0)
			return false;

		return true;
	}

	/** Calculates the distance between two particles
	 */
	private double distance(particle a, particle b){
		double dist = 0;
		for(int i = 0; i < dim; i++){
			dist += (Math.pow(a.position[i] - b.position[i], 2));
		}

		return Math.sqrt(dist);
	}

	/** Initializes the swarm
	 */
	private void init(){
		eval = 0;
 		test = new benchmark();
		function = test.testFunctionFactory(fun, dim);
		for(int i = 0; i < size; i++){
			swarm.add(new particle(dim, test.get_max_value(fun), test.get_min_value(fun)));
			swarm.get(i).init(rand);
			if(fun == 7 || fun == 25){
				swarm.get(i).max_bounds = Double.MAX_VALUE;
				swarm.get(i).min_bounds = -Double.MAX_VALUE;
			}
		}
		//init L
		OA();
	}

	/** Return the reference particle for a given particle according the type of the neighborhood
	 * @param pos Index of the particle
	 * @return The reference particle
	 */
	private particle reference_particle(int pos){
		switch(type){
			case 1: // local social
				int ini,end;
				ini = pos - neighbor / 2;
				end = pos + neighbor / 2;
				if(neighbor % 2 == 1)
					ini--;
				ini = (ini + swarm.size()) % swarm.size();
				end = end % swarm.size();

				particle lbest = swarm.get(ini);
				for(ini = (ini + 1) % swarm.size(); ini != end; ini = (ini + 1) % swarm.size())
					if(swarm.get(ini).fitness < lbest.fitness)
						lbest = swarm.get(ini);
				return lbest;
				
			case 2: // local geographic
				ArrayList<particle> nearest = new ArrayList<particle>(neighbor);
				ArrayList<Double> dist = new ArrayList<Double>(neighbor);
				for(int i = 0; i < neighbor; i++){
					dist.set(i, Double.MAX_VALUE);
				}
				//find the neighbor particles nearest to actual
				for(int i = 0; i < swarm.size(); i++){
					if (i != pos){
						double dist_aux = distance(swarm.get(pos),swarm.get(i));
						for(int j = 0; j < neighbor; j++){
							if(dist_aux < dist.get(j)){
								for(int k = neighbor-1; k > j; k--){
									dist.set(k, dist.get(k -1));
									nearest.set(k,nearest.get(k -1));
								}
								dist.set(j, dist_aux);
								nearest.set(j, swarm.get(i));
								break;
							}
						}
					}
				}

				//choose the best of the nearest
				int index = 0;
				double best = nearest.get(0).fitness;
				for(int i = 1; i < neighbor; i++){
					if(nearest.get(i).fitness < best){
						best = nearest.get(i).fitness;
						index = i;
					}
				}

				return nearest.get(index);

			default: // global
				return best_particle;
		}
	}

	/* Initializes the Orthogonal Array */
	private void OA(){
		//step 1
		int M = (int) Math.pow(2, Math.ceil(Math.log(dim+1)/Math.log(2)));
		int N = M-1;
		int u = (int) Math.ceil(Math.log(M)/Math.log(2));
		int b;
		
		L = new int[M][N];
		
		
		//step 2
		for(int a = 0; a < M; a++){
			for(int k = 0; k < u; k++){
				b = (int) Math.pow(2,k-1);
				L[a][b] = (int) (Math.floor((a-1)/Math.pow(2, u-k))+2) % 2;
			}
		}
		
		//step 3
		for(int a = 0; a < M; a++){
			for(int k = 1; k < u; k++){
				b = (int) Math.pow(2,k-1);
				for(int s = 0; s < b-1; s++){
					L[a][b+s] = (L[a][s] + L[a][b]) % 2;
				}
			}
		}
		//no need of step 4  % 2 already set vaues to 0 or 1
	}
	
	/** Create an artificial particle using the orthogonal experimental design
	 * @param a The particle
	 * @param b The reference particle
	 * @return The artificial particle for a
	 */
	private index_particle OED(particle a, particle b){
		/* Initialization using the Orthogonal Array */
		ArrayList<index_particle> F = new ArrayList<index_particle>();
		index_particle aux;
		for(int i = 0; i < L.length; i++){
			F.add(new index_particle(dim));
			aux = F.get(i);
			aux.index = L[i].clone();
		}
		
		/* Evaluate the artificial particles */
		int min = 0;
		double min_fit = Double.MAX_VALUE;
		for(int i = 0; i < F.size(); i++){
			F.get(i).fitness = function.f(F.get(i).get_positions(a, b));
			eval++;
			if(eval == max_evaluations)
				return F.get(i);
			if(F.get(i).fitness < min_fit){
				min_fit = F.get(i).fitness;
				min = i;
			}
		}
		
		/* Calculate the S values */
		double S[][] = new double[2][dim];
		double cont[][] = new double[2][dim];
		
		for(int i = 0; i < 2; i++)
			for(int j = 0; j < dim; j++)
				cont[i][j] = 0;
		
		for(int i = 0; i < F.size(); i++){
			for(int j = 0; j < dim; j++){
				S[F.get(i).index[j]][j] = F.get(i).fitness;
				cont[F.get(i).index[j]][j]++;
			}
		}
		
		for (int i = 0; i < 2; i++) 
			for (int j = 0; j < dim; j++) 
				S[i][j] = S[i][j] / cont[i][j];
		
		/* Constructing the final artificial particle */
		index_particle ret = new index_particle(dim);
		
		for (int j = 0; j < dim; j++)
			if(S[0][j] < S[1][j])
				ret.index[j] = 0;
			else
				ret.index[j] = 1;
		/* Evaluate the particle */
		ret.fitness = function.f(ret.get_positions(a, b));
		eval++;
		
		/* Choose if better */
		if(F.get(min).fitness < ret.fitness)
			ret = F.get(min);
		
		if(ret.fitness < best_particle.best_fitness){
			particle best = new particle(dim, test.get_max_value(fun), test.get_min_value(fun));
			best.position = ret.get_positions(a, b);
			best.best_position = ret.get_positions(a, b);
			best.fitness = ret.fitness;
			best.best_fitness = ret.fitness;
			best_particle = best;
		}
		
		return ret;
	}
	
	/** PSO Main function */
	public void run_pso(){
		/* Evaluates the initial population and find the first best*/
		best_particle = new particle();
		swarm.get(0).fitness = function.f(swarm.get(0).position);
		eval++;
		swarm.get(0).best_fitness = swarm.get(0).fitness;
		swarm.get(0).best_position = swarm.get(0).position.clone();
		best_particle.best_fitness = swarm.get(0).best_fitness;
		best_particle.best_position = swarm.get(0).best_position.clone();

		for(int i = 1; i < swarm.size() && eval < max_evaluations; i++){
			swarm.get(i).fitness = function.f(swarm.get(i).position);
			eval++;
			swarm.get(i).best_fitness = swarm.get(i).fitness;
			swarm.get(i).best_position = swarm.get(i).position.clone();
			if(swarm.get(i).best_fitness < best_particle.best_fitness)
				best_particle = swarm.get(i);
		}

		error = best_particle.best_fitness - function.bias();
		particle.update_intertia(eval, max_evaluations);

		/* PSO main boucle*/
		while(eval < max_evaluations && error > Math.pow(10, -8)){
			/* Updates particles' velocity */
			for(int i = 0; i < swarm.size() && eval < max_evaluations; i++){
				particle p = reference_particle(i);
				/* If the solution doesn't updates in mas_gen generations */
				if(swarm.get(i).generations < 0){
					/* Use the Orthogonal Experimental Design to set the artificial reference particle */
					swarm.get(i).real_ref = OED(swarm.get(i), p);
					swarm.get(i).generations = max_gen;
				}
				swarm.get(i).update_velocity(p, rand);
			}

			/* Updates particles' position */
			for(int i = 0; i < swarm.size() && eval < max_evaluations; i++){
				swarm.get(i).update_position();
				/* Evaluates new positions*/
				swarm.get(i).fitness = function.f(swarm.get(i).position);
				eval++;
				
				/* Check for own best */
				if(swarm.get(i).fitness < swarm.get(i).best_fitness){
					swarm.get(i).best_fitness = swarm.get(i).fitness;
					swarm.get(i).best_position = swarm.get(i).position.clone();
					swarm.get(i).generations = max_gen; // restart generations counter
				}
				
				/* Check for best particle*/
				if(swarm.get(i).fitness < best_particle.best_fitness)
					best_particle = swarm.get(i);
				
				swarm.get(i).generations--;
			}
			error = best_particle.best_fitness - function.bias();
			/* Inertia update */
			particle.update_intertia(eval, max_evaluations);
		}
	}

	/** Print results
	 */
	public void print_result(){
		System.out.println(seed + "," + fun + "," + function.bias() + ","+
				best_particle.best_fitness + "," + error + "," + eval);
	}

	/** Main function
	 */
	static public void main(String[] args){
		olpso PSO = new olpso();
		if(!PSO.check_parameters(args))
			PSO.print_usage();

		PSO.init();

		PSO.run_pso();

		PSO.print_result();
	}

}
