package Selection_PSO;

/*
	pso.java
	Pablo David Gutiérrez Pérez.
	
	Copyright (c) 2011.

*/

import cec05.benchmark;
import cec05.test_func;
import java.util.Random;
import java.util.ArrayList;

/**
 *
 * @author Pablo David Gutiérrez Pérez
 */
public class pso {
	/** Swarm of particles */
	protected ArrayList<particle> swarm;
	/** Number of evaluations spent */
	protected int eval;
	/** Maximum evaluations */
	protected int max_evaluations;
	/** Problem's dimension */
	protected int dim;
	/** Number of the function */
	protected int fun;
	/** Swarm size */
	protected int size;
	/** Best particle */
	protected particle best_particle;
	/** Random generator */
	protected Random rand;
	/** Type of neighborhood */
	protected int type;
	/** Size of neighborhood */
	protected int neighbor;
	/** Test function */
	protected benchmark test;
	protected test_func function;
	/** Seed of random generator */
	protected long seed;
	/** Best error */
	protected double error;
	/** Number of iterations */
	protected int iter;
	public pso(){iter = 0;}

	/** Function that prints usage
	 */
	private void print_usage(){
		System.out.println("Params error");
		System.out.println("Usage: java -cp dist/PSO.jar Selection_PSO.pso function dimension seed population type neighbor");
		System.out.println("Type 0 -> global, doesn't need neighbor param");
		System.out.println("Type 1 -> local social");
		System.out.println("Type 2 -> local geographical");
		System.exit(-1);
	}

	/** Checks the parameters value
	 */
	private boolean check_parameters(String[] params){
		if(params.length != 5 && params.length != 6)
			return false;
		int i = 0;
		//Function
		fun = Integer.parseInt(params[i]);
		if(fun <= 0 || fun > 25)
			return false;
		i++;
		//Dimension
		dim = Integer.parseInt(params[i]);
		if(dim <= 0 || dim > 50)
			return false;
		i++;
		//Number of evaluations
		max_evaluations = 10000*dim;
		//Seed
		seed =  Long.parseLong(params[i]);
		rand = new Random(seed);
		i++;
		//Swarm size
		size = Integer.parseInt(params[i]);
		if(size <= 0)
			return false;
		swarm = new ArrayList<particle>(size);
		i++;
		//type of algorithm
		// 0 -> global
		// 1 -> social
		// 2 -> geographical
		type = Integer.parseInt(params[i]);
		if(type < 0 || type >= 3)
			return false;
		i++;
		if(type > 0){
			if(params.length == 5)
				return false;
			neighbor = Integer.parseInt(params[i]);
			if(neighbor <= 0 || neighbor > size)
				return false;
		}

		return true;
	}

	/** Calculates the distance between two particles
	 */
	private double distance(particle a, particle b){
		double dist = 0;
		for(int i = 0; i < dim; i++){
			dist += (Math.pow(a.position[i] - b.position[i], 2));
		}

		return Math.sqrt(dist);
	}

	/** Initializes the swarm
	 */
	private void init(){
		eval = 0;
 		test = new benchmark();
		function = test.testFunctionFactory(fun, dim);
		for(int i = 0; i < size; i++){
			swarm.add(new particle(dim, test.get_max_value(fun), test.get_min_value(fun)));
			swarm.get(i).init(rand);
			if(fun == 7 || fun == 25){
				swarm.get(i).max_bounds = 999999999;
				swarm.get(i).min_bounds = -999999999;
			}
		}
	}

	/** Return the reference particle for a given particle according the type of the neighborhood
	 * @param pos Index of the particle
	 * @return The reference particle
	 */
	private particle reference_particle(int pos){
		switch(type){
			case 1: // local social
				int ini,end;
				ini = pos - neighbor / 2;
				end = pos + neighbor / 2;
				if(neighbor % 2 == 1)
					ini--;
				ini = (ini + swarm.size()) % swarm.size();
				end = end % swarm.size();

				particle lbest = swarm.get(ini);
				for(ini = (ini + 1) % swarm.size(); ini != end; ini = (ini + 1) % swarm.size())
					if(swarm.get(ini).fitness < lbest.fitness)
						lbest = swarm.get(ini);
				return lbest;

			case 2: // local geographic
				particle nearest[] = new particle[neighbor];
				double dist[] = new double[neighbor];
				for(int i = 0; i < neighbor; i++){
					dist[i] = 99999999;
				}
				//find the neighbor particles nearest to actual
				for(int i = 0; i < swarm.size(); i++){
					if (i != pos){
						double dist_aux = distance(swarm.get(pos),swarm.get(i));
						for(int j = 0; j < neighbor; j++){
							if(dist_aux < dist[j]){
								for(int k = neighbor-1; k > j; k--){
									dist[k] = dist[k -1];
									nearest[k] = nearest[k -1];
								}
								dist[j] = dist_aux;
								nearest[j] = swarm.get(i);
								break;
							}
						}
					}
				}

				//choose the best of the nearest
				int index = 0;
				double best = nearest[0].fitness;
				for(int i = 1; i < neighbor; i++){
					if(nearest[i].fitness < best){
						best = nearest[i].fitness;
						index = i;
					}
				}

				return nearest[index];

			default: // global
				return best_particle;
		}
	}

	/** Apply the selection operator, the operator sort the swarm and discard
	 * the particles with worse fitness, cloning the best ones.
	 */
	public void selecction(){
		ArrayList<particle> sorted = new ArrayList<particle>();
		boolean cont = true;
		/* Sort the swarm */
		for(int i = 0; i < swarm.size(); i++){
			cont = true;
			for(int j = 0; j < sorted.size() && cont; j++){
				if(swarm.get(i).fitness < sorted.get(j).fitness){
					cont = false;
					sorted.add(j,swarm.get(i));
				}
			}
			if(cont == true)
				sorted.add(swarm.get(i));
		}
		/* Discard the worse particles*/
		int init, discard;
		init = 0;
		discard = swarm.size()/2;
		iter++;
		
		/* Clone the best particles */
		for(; discard < swarm.size(); discard++, init++){
			sorted.get(discard).fitness = sorted.get(init).fitness;
			sorted.get(discard).position = sorted.get(init).position.clone();
			sorted.get(discard).velocity = sorted.get(init).velocity.clone();
			if(sorted.get(discard).fitness < sorted.get(init).best_fitness){
				sorted.get(discard).best_fitness = sorted.get(init).fitness;
				sorted.get(discard).best_position = sorted.get(init).position.clone();
			}
		}

	}

	/** PSO Main function */
	public void run_pso(){
		/* Evaluates the initial population and find the first best*/
		best_particle = new particle();
		swarm.get(0).fitness = function.f(swarm.get(0).position);
		eval++;
		swarm.get(0).best_fitness = swarm.get(0).fitness;
		swarm.get(0).best_position = swarm.get(0).position;
		best_particle.best_fitness = swarm.get(0).best_fitness;
		best_particle.best_position = swarm.get(0).best_position.clone();

		for(int i = 1; i < swarm.size() && eval < max_evaluations; i++){
			swarm.get(i).fitness = function.f(swarm.get(i).position);
			eval++;
			swarm.get(i).best_fitness = swarm.get(i).fitness;
			swarm.get(i).best_position = swarm.get(i).position.clone();
			if(swarm.get(i).best_fitness < best_particle.best_fitness)
				best_particle = swarm.get(i);
		}

		error = best_particle.best_fitness - function.bias();
		particle.update_intertia(eval, max_evaluations);
		/* Using selecction */
		selecction();

		/* PSO main boucle*/
		while(eval < max_evaluations && error > Math.pow(10, -8)){
			/* Updates particles' velocity */
			for(int i = 0; i < swarm.size(); i++){
				swarm.get(i).update_velocity(reference_particle(i), rand);
			}

			/* Updates particles' position */
			for(int i = 0; i < swarm.size() && eval < max_evaluations; i++){
				swarm.get(i).update_position();
				/* Evaluates new positions*/
				swarm.get(i).fitness = function.f(swarm.get(i).position);
				eval++;
				
				/* Check for own best */
				if(swarm.get(i).fitness < swarm.get(i).best_fitness){
					swarm.get(i).best_fitness = swarm.get(i).fitness;
					swarm.get(i).best_position = swarm.get(i).position.clone();
				}
				
				/* Check for best particle*/
				if(swarm.get(i).fitness < best_particle.best_fitness)
					best_particle = swarm.get(i);
			}
			error = best_particle.best_fitness - function.bias();
			particle.update_intertia(eval, max_evaluations);
			/* Using selecction */
			selecction();
		}
	}

	/** Print results
	 */
	public void print_result(){
		System.out.println(seed + "," + fun + "," + function.bias() + ","+
				best_particle.best_fitness + "," + error + "," + eval);
	}

	/** Main function
	 */
	static public void main(String[] args){
		pso PSO = new pso();
		if(!PSO.check_parameters(args))
			PSO.print_usage();

		PSO.init();

	  long t0 = System.currentTimeMillis();
		PSO.run_pso();
	  long t1 = System.currentTimeMillis();
	  
	  System.out.println(t1-t0);

		PSO.print_result();
	}

}
