package Stretching_PSO;

/*
	pso.java
	Pablo David Gutiérrez Pérez.
	
	Copyright (c) 2011.

*/

import cec05.benchmark;
import cec05.test_func;
import java.util.ArrayList;
import java.util.Random;

/**
 *
 * @author Pablo David Gutiérrez Pérez
 */
public class pso {
	static protected double gamma_1 = 5000;
	static protected double gamma_2 = 0.5;
	static protected double mu = Math.pow(10, -10);
	/** Swarm of particles */
	protected ArrayList<particle> swarm;
	/** Number of evaluations spent */
	protected int eval;
	/** Maximum evaluations */
	protected int max_evaluations;
	/** Problem's dimension */
	protected int dim;
	/** Number of the function */
	protected int fun;
	/** Swarm size */
	protected int size;
	/** Best particle */
	protected particle best_particle;
	/** Random generator */
	protected Random rand;
	/** Type of neighborhood */
	protected int type;
	/** Size of neighborhood */
	protected int neighbor;
	/** Test function */
	protected benchmark test;
	protected test_func function;
	/** Seed of random generator */
	protected long seed;
	/** Best error */
	protected double error;
	/** Local minimum position */
	private double[] local_minima;
	/** Local minimum fitness */
	private double local_minima_fitness = Double.MAX_VALUE;
	/** Global minimum position */
	private double[] global_minima;
	/** Global minimum fitness */
	private double global_minima_fitness = Double.MAX_VALUE;

	public pso(){}

	/** Function that prints usage
	 */
	private void print_usage(){
		System.out.println("Params error");
		System.out.println("Usage: java -cp dist/PSO.jar Stretching_PSO.pso function dimension seed population type neighbor");
		System.out.println("Type 0 -> global, doesn't need neighbor param");
		System.out.println("Type 1 -> local social");
		System.out.println("Type 2 -> local geographical");
		System.exit(-1);
	}
	
	/** Checks the parameters value
	 */
	private boolean check_parameters(String[] params){
		if(params.length != 5 && params.length != 6)
			return false;
		int i = 0;
		//Function
		fun = Integer.parseInt(params[i]);
		if(fun <= 0 || fun > 25)
			return false;
		i++;
		//Dimension
		dim = Integer.parseInt(params[i]);
		if(dim <= 0 || dim > 50)
			return false;
		i++;
		//Number of evaluations
		max_evaluations = 10000*dim;
		//Seed
		seed =  Long.parseLong(params[i]);
		rand = new Random(seed);
		i++;
		//Swarm size
		size = Integer.parseInt(params[i]);
		if(size <= 0)
			return false;
		swarm = new ArrayList<particle>(size);
		i++;
		//type of algorithm
		// 0 -> global
		// 1 -> social
		// 2 -> geographical
		type = Integer.parseInt(params[i]);
		if(type < 0 || type >= 3)
			return false;
		i++;
		if(type > 0){
			if(params.length == 5)
				return false;
			neighbor = Integer.parseInt(params[i]);
			if(neighbor <= 0 || neighbor > size)
				return false;
		}

		return true;
	}

	/** Calculates the distance between two particles
	 */
	private double distance(particle a, particle b){
		double dist = 0;
		for(int i = 0; i < dim; i++){
			dist += (Math.pow(a.position[i] - b.position[i], 2));
		}

		return Math.sqrt(dist);
	}

	/** Initializes the swarm
	 */
	private void init(){
		eval = 0;
 		test = new benchmark();
		function = test.testFunctionFactory(fun, dim);
		for(int i = 0; i < size; i++){
			swarm.add(new particle(dim, test.get_max_value(fun), test.get_min_value(fun)));
			swarm.get(i).init(rand);
			if(fun == 7 || fun == 25){
				swarm.get(i).max_bounds = 999999999;
				swarm.get(i).min_bounds = -999999999;
			}
		}
	}

	/** Return the reference particle for a given particle according the type of the neighborhood
	 * @param pos Index of the particle
	 * @return The reference particle
	 */
	private particle reference_particle(int pos){
		switch(type){
			case 1: // local social
				int ini,end;
				ini = pos - neighbor / 2;
				end = pos + neighbor / 2;
				if(neighbor % 2 == 1)
					ini--;
				ini = (ini + swarm.size()) % swarm.size();
				end = end % swarm.size();

				particle lbest = swarm.get(ini);
				for(ini = (ini + 1) % swarm.size(); ini != end; ini = (ini + 1) % swarm.size())
					if(swarm.get(ini).fitness < lbest.fitness)
						lbest = swarm.get(ini);
				return lbest;
				
			case 2: // local geographic
				particle nearest[] = new particle[neighbor];
				double dist[] = new double[neighbor];
				for(int i = 0; i < neighbor; i++){
					dist[i] = 99999999;
				}
				//find the neighbor particles nearest to actual
				for(int i = 0; i < swarm.size(); i++){
					if (i != pos){
						double dist_aux = distance(swarm.get(pos),swarm.get(i));
						for(int j = 0; j < neighbor; j++){
							if(dist_aux < dist[j]){
								for(int k = neighbor-1; k > j; k--){
									dist[k] = dist[k -1];
									nearest[k] = nearest[k -1];
								}
								dist[j] = dist_aux;
								nearest[j] = swarm.get(i);
								break;
							}
						}
					}
				}

				//choose the best of the nearest
				int index = 0;
				double best = nearest[0].fitness;
				for(int i = 1; i < neighbor; i++){
					if(nearest[i].fitness < best){
						best = nearest[i].fitness;
						index = i;
					}
				}

				return nearest[index];

			default: // global
				return best_particle;
		}
	}

	/** Function sign of a number
	 * @param x The number
	 */
	private double sign(double x){
		if(x == 0)
			return 0;
		else if(x < 0)
			return -1;
		return 1;
	}

	/** Calculates de G function of the stretching
	 * @param x The position to evaluate
	 * @param f The usual fitness of x
	 * @return The G function value
	 */
	private double stretched_g(double[] x, double f){
		double ret;
		ret = f + gamma_1 * norm(x)* (sign(f - local_minima_fitness)+1);
		return ret;
	}

	/** Calculates the norm
	 * @param x The position to evaluate
	 * @return The norm between the position and the local minimum
	 */
	private double norm(double[] x){
		double ret = 0;
		for(int i = 0; i < x.length; i++){
			ret = Math.pow((x[i] - local_minima[i]), 2);
		}
		ret = Math.sqrt(ret);
		return ret;
	}

	/** Evaluation function
	 * @param x The position to evaluate
	 * @return the usual fitness value, if the search haven't been stuck,
	 * or the stretched fitness, otherwise.
	 */
	private double stretched_function(double[] x){
		if(local_minima_fitness == Double.MAX_VALUE)
			return function.f(x);
		
		double f = function.f(x);
		double ret;
		ret = stretched_g(x, f) + gamma_2 * (
				(sign(f - local_minima_fitness)+1)/
				(Math.tanh(mu * (stretched_g(x, f) - stretched_g(local_minima, local_minima_fitness)))));
		return ret;

	}

	/** PSO Main function */
	public void run_pso(){
		int eval1 = 0;
		/* Evaluates the initial population and find the first best*/
		best_particle = new particle();
		swarm.get(0).fitness = function.f(swarm.get(0).position);
		eval++;
		swarm.get(0).best_fitness = swarm.get(0).fitness;
		swarm.get(0).best_position = swarm.get(0).position;
		best_particle.best_fitness = swarm.get(0).best_fitness;
		best_particle.best_position = swarm.get(0).best_position.clone();

		for(int i = 1; i < swarm.size() && eval < max_evaluations; i++){
			swarm.get(i).fitness = function.f(swarm.get(i).position);
			eval++;
			swarm.get(i).best_fitness = swarm.get(i).fitness;
			swarm.get(i).best_position = swarm.get(i).position.clone();
			if(swarm.get(i).best_fitness < best_particle.best_fitness)
				best_particle = swarm.get(i);
		}

		error = best_particle.best_fitness - function.bias();
		particle.update_intertia(eval, max_evaluations);
		int counter = 0;

		/* PSO main boucle*/
		while(eval < max_evaluations && error > Math.pow(10, -8)){
			/* Updates particles' velocity */
			for(int i = 0; i < swarm.size(); i++){
				swarm.get(i).update_velocity(reference_particle(i), rand);
			}

			/* Updates particles' position */
			for(int i = 0; i < swarm.size() && eval < max_evaluations; i++){
				swarm.get(i).update_position();
				/* Evaluates new positions*/
				swarm.get(i).fitness = stretched_function(swarm.get(i).position);
				eval++;
				
				/* Check for own best */
				if(swarm.get(i).fitness < swarm.get(i).best_fitness){
					swarm.get(i).best_fitness = swarm.get(i).fitness;
					swarm.get(i).best_position = swarm.get(i).position.clone();
				}
			}
			counter ++;
			/* Check for best particle*/
			for(int i = 0; i < swarm.size(); i++)
				if(swarm.get(i).best_fitness < best_particle.best_fitness){
					best_particle = swarm.get(i);
					counter = 0;
				}
			
			/* If the search is stuck set stretching parameters */
			if(counter > (1000*dim)/swarm.size() ){
				eval1=eval;
				local_minima = best_particle.best_position.clone();
				local_minima_fitness = best_particle.best_fitness;
				if(best_particle.best_fitness < global_minima_fitness){
					global_minima = best_particle.best_position.clone();
					global_minima_fitness = best_particle.best_fitness;
				}
				counter = 0;
				/* Restart own best */
				best_particle.best_fitness = Double.MAX_VALUE;
				for(int i = 0; i < swarm.size(); i++){
					swarm.get(i).best_fitness = Double.MAX_VALUE;
				}
			}
			error = best_particle.best_fitness - function.bias();
			/* Inertia update */
			particle.update_intertia(eval, max_evaluations-eval1);
		}
	}

	/** Print results
	 */
	public void print_result(){
		System.out.print(seed + "," + fun + "," + function.bias() + ",");
		if(global_minima_fitness < best_particle.best_fitness){
			System.out.print(global_minima_fitness);
			error = global_minima_fitness - function.bias();
		}
		else{
			System.out.print(best_particle.best_fitness);
			error = best_particle.best_fitness - function.bias();
		}
		System.out.println("," + error + "," + eval);
	}

	/** Main function
	 */
	static public void main(String[] args){
		pso PSO = new pso();
		if(!PSO.check_parameters(args))
			PSO.print_usage();

		PSO.init();

		PSO.run_pso();

		PSO.print_result();
	}

}
